use std::iter::repeat;

use crate::{game::chunk::Block, vec3, vector::Vector3};

use super::Mesh;

const TOP_POSITIONS: [Vector3<f32>; 6] = [
    vec3!(0.0, 1.0, 1.0),
    vec3!(1.0, 1.0, 0.0),
    vec3!(0.0, 1.0, 0.0),
    vec3!(1.0, 1.0, 1.0), // 3
    vec3!(1.0, 1.0, 0.0), // 2
    vec3!(0.0, 1.0, 1.0), // 1
];

const TOP_NORMAL: Vector3<f32> = vec3!(0.0, 1.0, 0.0);

const BOTTOM_POSITIONS: [Vector3<f32>; 6] = [
    vec3!(0.0, 0.0, 0.0),
    vec3!(1.0, 0.0, 0.0),
    vec3!(0.0, 0.0, 1.0),
    vec3!(0.0, 0.0, 1.0), // 1
    vec3!(1.0, 0.0, 0.0), // 2
    vec3!(1.0, 0.0, 1.0), // 3
];

const BOTTOM_NORMAL: Vector3<f32> = vec3!(0.0, -1.0, 0.0);

const NORTH_POSITIONS: [Vector3<f32>; 6] = [
    vec3!(1.0, 0.0, 0.0),
    vec3!(1.0, 1.0, 0.0),
    vec3!(1.0, 0.0, 1.0),
    vec3!(1.0, 1.0, 1.0),
    vec3!(1.0, 0.0, 1.0),
    vec3!(1.0, 1.0, 0.0),
];

const NORTH_NORMAL: Vector3<f32> = vec3!(1.0, 0.0, 0.0);

const SOUTH_POSITIONS: [Vector3<f32>; 6] = [
    vec3!(0.0, 0.0, 1.0),
    vec3!(0.0, 1.0, 0.0),
    vec3!(0.0, 0.0, 0.0),
    vec3!(0.0, 1.0, 1.0), // 3
    vec3!(0.0, 1.0, 0.0), // 2
    vec3!(0.0, 0.0, 1.0), // 1
];

const SOUTH_NORMAL: Vector3<f32> = vec3!(-1.0, 0.0, 0.0);

const EAST_POSITIONS: [Vector3<f32>; 6] = [
    vec3!(0.0, 0.0, 1.0),
    vec3!(1.0, 0.0, 1.0),
    vec3!(0.0, 1.0, 1.0),
    vec3!(0.0, 1.0, 1.0),
    vec3!(1.0, 0.0, 1.0),
    vec3!(1.0, 1.0, 1.0),
];

const EAST_NORMAL: Vector3<f32> = vec3!(0.0, 0.0, 1.0);

const WEST_POSITIONS: [Vector3<f32>; 6] = [
    vec3!(0.0, 1.0, 0.0),
    vec3!(1.0, 0.0, 0.0),
    vec3!(0.0, 0.0, 0.0),
    vec3!(1.0, 1.0, 0.0),
    vec3!(1.0, 0.0, 0.0),
    vec3!(0.0, 1.0, 0.0),
];

const WEST_NORMAL: Vector3<f32> = vec3!(0.0, 0.0, -1.0);

const POSITIONS: [&'static [Vector3<f32>; 6]; 6] = [
    &TOP_POSITIONS,
    &BOTTOM_POSITIONS,
    &NORTH_POSITIONS,
    &SOUTH_POSITIONS,
    &EAST_POSITIONS,
    &WEST_POSITIONS,
];

const NORMALS: [&'static Vector3<f32>; 6] = [
    &TOP_NORMAL,
    &BOTTOM_NORMAL,
    &NORTH_NORMAL,
    &SOUTH_NORMAL,
    &EAST_NORMAL,
    &WEST_NORMAL,
];

pub fn mesh_block(mesh: &mut Mesh, block: &Block, pos: Vector3<i32>, sides: [bool; 6]) {
    let pos = Into::<Vector3<f32>>::into(pos);
    mesh.positions.extend(
        POSITIONS
            .iter()
            .copied()
            .zip(sides)
            .filter_map(|(s, e)| e.then(|| s.iter().map(|p| *p + pos)))
            .flatten(),
    );
    mesh.normals.extend(
        NORMALS
            .iter()
            .copied()
            .zip(sides)
            .filter_map(|(s, e)| e.then(|| repeat(s).take(6)))
            .flatten(),
    );
}
